********************************************************
*       Equates used for masks and ID
********************************************************
* Serial mask equates
rate    equ     %00110000       Set Baud Rate to 9600 Baud 
sccr1   equ     %00000000       Set SCI reg 1 
sccr2   equ     %00101100       Set SCI reg 2  to receive
sccr2b  equ     %00001100       Set SCI reg 2  to receive
reset   equ     %11111111       Reset for data register

*Message Equates for communications with PC

*Recieve command equates
*                FEDCBA9876543210
bdwdth  equ     %01010000               width of board to process
stmsg   equ     %01100000               indicates how many messages to follow


*Transmit command equates
*                FEDCBA9876543210
numall  equ     %10000000              number of commands allowed
prgack  equ     %0000000000000001       Acknowledge progress indicator
somack  equ     %0000000000000010       Acknowledge start of messages
endopt  equ     %0000000000000011       Last operation executed
xmaxlim equ     %0000000000000100       Limiter hit x-max
xminlim equ     %0000000000000101       Limiter hit x-min
ymaxlim equ     %0000000000000110       Limiter hit y-max
yminlim equ     %0000000000000111       Limiter hit y-min
invrcv  equ     %0000000000001000       did not receive all messages successfully
ywahmsg equ     %0000000000001001       establish com link message
succrcv equ     %0000000000001010       received all messages successfully
calsuc  equ     %0000000000001011       machine calibrated sucessfully

********************************************************
*       Equates for use with index offset of = $1000
********************************************************
* Port equates
PortA   equ     $00
PortD   equ     $08
PortE   equ     $0A
DDRD    equ     $09
* Serial Communications Equates
BAUD    equ     $2B     BAUD RATE
SCCR1   equ     $2C     SCI CONTROL REG 1
SCCR2   equ     $2D     SCI CONTROL REG 2
SCSR    equ     $2E     SCI STATUS REG
SCDR    equ     $2F     SCI DATA REG                
CONFIG  equ     $3f     CONFIG REG              

********************************************************
*       Movement Equates for Port D
********x********x*******x*******x**********************
*                76543210
headu   equ     %00010000       Put head up      //bit clear
headd   equ     %00010000       Put head down    //bit set
mover   equ     %00100000       Move head right  //bit clear
movel   equ     %00100000       Move head left   //bit set
moveb   equ     %00001000       Move head back   //bit clear
movef   equ     %00001000       Move head forward//bit set
lduf    equ     %00101000       Calibrate head
*
********************************************************
*       Limit Equates for Port E
********x********x*******x*******x**********************
*                76543210
liml    equ     %00000001       Left limiter
limr    equ     %00000010       Right limiter
limf    equ     %00000100       Front limiter
limb    equ     %00001000       Back limiter
limall  equ     %00001111       All limiters
*
********************************************************
*       Movement Equates for Port A
********x********x*******x*******x**********************
*                76543210
startx  equ     %01000000       Start X movement //bit clear
stopx   equ     %01000000       Stop X movement  //bit set
starty  equ     %00100000       Start Y movement //bit clear
stopy   equ     %00100000       Stop Y movement  //bit set
stopxy  equ     %01100000       Stop both xy movement
*
********************************************************
*       Generic Equates 
********x*******x*******x*******x***********************
stack    equ    $01FF   Stack location
stack2   equ    $B5FF   Stack 2 location
recv_vec equ    $FFD6   Receive Vector
stck_sz  equ    $52DA   Stack Size
********************************************************
*       Flag Mask
********x*******x*******x*******x***********************
*                76543210
rcv_buf1 equ    %00000001       Bit is to indicate whether the first
*                              half of the receive buffer has been received
rcv_buf2 equ    %00000010       Bit is to indicate whether the second 
*                              half of the receive buffer has been received

********************************************************
*       Operation Instruction Mask
********x*******x*******x*******x***********************
*                76543210
head     equ    %01000000       determine head position
movement equ    %00011000       determine movement
right    equ    %00000000
left     equ    %00010000
forward  equ    %00001000
backward equ    %00011000
*
milval   equ     9070
*
********************************************************
*       Receive Mask
********x*******x*******x*******x***********************
*               76543210
op_msk  equ    %10000000
bs_msk  equ    %01000000
*

********************************************************
*       Author: Keyur Khambholja
*       Name: Main
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
main            org $D000

	lds     #stack          Set the stack pointer
	
	sei                     Inhibit all interrupts

;                               The code on the folowing 4 lines is for 
;                               bootstrap only.  Don't remove comments unless
;                               memory definitions are changed for stack.
;       ldaa    #$7E
;       staa    $C4
;       ldx     #rcv_int
;       stx     $C5
	

	ldx     #$1000          Init x for index to registers

	ldaa    #rate           Set the baud rate to 9600
	staa    BAUD,x

	ldaa    #sccr2          Setup SCI control reg 2
	staa    SCCR2,x

	ldd     #rcv_int        Set receive interrupt vector
	std     recv_vec 
	
	ldaa    #sccr1          Setup SCI control reg 1
	staa    SCCR1,x

	ldd     #stack2         Load the second stack pointer
	std     Stack2       
		
	ldaa    #%111000       Set the direction bits for port D
	staa    DDRD,x
		      
	bset    PortA,x %01100000       Turn the dang motors off
	bclr    PortD,x %011000         
	
	
	clr     Recv        
	clr     Recv+1
	clr     Flag            Clear all bits of test Flag
	clr     Flag+1
	clr     Counter         Clear counter that indicates what byte has been received
	clr     Counter+1

	ldaa    SCSR,x
	ldaa    SCDR,x          Read RDR and store in low

loop    cli                     Reenable interrupts
	wai                     Wait for interrupt

	sei                     Inhibit all interrupts
	ldd     Flag            Check to see if byte received was first or second
	andb    #%00000011
	cmpb    #$3
	bne     loop

	ldd     Flag            Clear flag to indicate no bytes recieved
	andb    #%11111100
	stab    Flag+1

	ldaa    Recv            Load upper half of Receive message
	anda    #op_msk         Check to see if message received was an operation instruction
	beq     bs              
	ldd     Recv
	jsr     Store           Store onto the stack
	bra     loop

bs      ldaa    Recv            
	anda    #bs_msk
	beq     com

	ldaa    Recv
	anda    #%11110000
	cmpa    #bdwdth
	beq     num

	jsr     Stmsg
	bra     loop

num     jsr     Bdwdth
	bra     loop

com     ldab    Recv
	ldy     #recv_tb
	aby
	aby
	ldy     0,y
	jsr     0,y
	bra     loop
;End main 
********************************************************
*       Author: Keyur Khambholja
*       Name: Error
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Error
	psha    
	pshb

	ldd     Flag            Clear flag to indicate no bytes recieved
	andb    #%11111100
	stab    Flag+1

	ldd     #invrcv
	std     Trans
	jsr     Tx

	pulb
	pula

	rts
;end Error
********************************************************
*       Author: Keyur Khambholja
*       Name: Cancel
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Cancel
	psha
	pshb
	ldd     Flag
	andb    #%00000100
	clra
	std     Flag
	ldd     #stack2
	std     Stack2
	clr     Counter
	clr     Counter+1
	pulb
	pula
	rts
;Cancel
********************************************************
*       Author: Keyur Khambholja
*       Name: Manual
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Manual
	psha
	pshb
	ldab    Flag+1
	andb    #%00001100
	cmpb    #$0c
	beq     check2
	jsr     Inval
	bra     end2
check2  ldab    Flag+1
	orab    #%00101100
	stab    Flag+1
end2    pulb
	pula
	rts
;end Manual
********************************************************
*       Author: Keyur Khambholja
*       Name: Auto
*       Description: 
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Auto    psha
	pshb
	ldab    Flag+1
	andb    #%00001100
	cmpb    #$0c
	beq     check
	jsr     Inval
	bra     end
check   ldab    Flag+1
	orab    #%00011100
	stab    Flag+1
end     pulb
	pula
	rts
;end Auto
********************************************************
*       Author: Keyur Khambholja
*       Name: Are you there message
*       Description: Routine establishes there is a communications link
*       PreConditions:  This routine acknowledges that we are ready 
*                       to receive messages
*       PostCondions: None
*       Registers Modified: A,B
*       Calls: Tx
********x*******x*******x*******x***********************
Aytmsg  psha
	pshb
	ldaa    #%00000100
	staa    Flag+1
	ldd     #ywahmsg
	std     Trans
	jsr     Tx
	pulb
	pula
	rts
;end Aytmsg
********************************************************
*       Author: Keyur Khambholja
*       Name: Start Transmission
*       Description:    This routine acknowledges that we are ready 
*                       to receive messages
*       PreConditions:  The receive interrupt must be disabled
*       PostCondions:   None
*       Registers Modified: A, B
*       Calls: Tx, Inval
********x*******x*******x*******x***********************
Sttrm
	psha                    Push registers used
	pshb
*       
	ldab    Flag+1          Check to see if AYT already received
	andb    #%00000100
	cmpb    #$4
	beq     sttrma          If ayt received then transmit somack
	jsr     Inval           If ayt not received the jmp to invalid message
	bra     sttrmb
*
sttrma  ldd     #somack         Load ack into transmit buffer
	std     Trans
	jsr     Tx
	clr     Flag+1
	clr     Counter
	clr     Counter+1
	ldd     #stack2
	std     Stack2
	ldaa    Flag+1
	oraa    #%00001100
	staa    Flag+1
*
sttrmb  pulb                    Pull registers off of the stack
	pula
	rts

;end Sttrm
********************************************************
*       Author: Keyur Khambholja
*       Name: Endmsg
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Endmsg
	psha
	pshb
	ldd     Mes_num
	cpd     Counter
	beq     correct
	jsr     Inval
	ldd     #invrcv
	std     Trans
	jsr     Tx
	bra     go
correct ldd     #succrcv
	std     Trans
	jsr     Tx
	jsr     Process
go      pulb
	pula
	rts
;End Endmsg
********************************************************
*       Author: Keyur Khambholja
*       Name: Progrcv
*       Description: Progress message to update PC
*       PreConditions: Message Progrcv remain in Recv buffer
*       PostCondions: Message pushed onto stack
*       Registers Modified: None
*       Calls: Store
********x*******x*******x*******x***********************
Progrcv
	jsr     Store   Store the message in the receive
*                       buffer onto the stack
	rts
;end Progrcv
********************************************************
*       Author: Keyur Khambholja
*       Name: Endopr
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Endopr
	jsr     Store   This message is placed on the stack and after
*                       And once reached an ack is sent
	rts

;end Endopr
********************************************************
*       Author: Keyur Khambholja
*       Name: Calibrate
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Calib
	psha
	pshb
	pshx
	pshy
		
	ldab    Flag+1
	orab    #%01000000
	stab    Flag+1

	ldx     #$1000
*
	bset    PortA,x %01100000       Turn motors off
*
	bclr    PortD,x %011000         Set to move lower left
	bset    PortD,x %100000
*
	bclr    PortA,x %00100000       Turn x motor on 
	brset   PortE,x liml *
	bset    PortA,x %00100000
*
	bclr    PortA,x %01000000          Turn y motor on
	brset   PortE,x limf *
	bset    PortA,x %01000000
*
	bclr    PortD,x %100000
	bclr    PortA,x %00100000       Turn x motor on 
	ldy     #900
	pshx

loopt   ldx     #7251           Move to center of the board
	jsr     Twomil
	dey
	bne     loopt
	pulx
	jsr     Motoroff        

	ldd     #calsuc         Successfully callibrated
	std     Trans
	jsr     Tx

	puly
	pulx
	pulb
	pula

	rts

;End Calib
********************************************************
*       Author: Keyur Khambholja
*       Name: Eject
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Eject
	pshx
	ldx     #$1000
*
	bset    PortA,x %01100000       Turn motors off
*
	bclr    PortD,x %111000         Set to upper right
	bset    PortD,x %001000
*
	bclr    PortA,x %00100000       Turn x motor on 
	brset   PortE,x limr *
	bset    PortA,x %00100000
*
	bclr    PortA,x %01000000          Turn y motor on
	brset   PortE,x limb *
	bset    PortA,x %01000000
*
	pulx
	rts
;end Eject
********************************************************
*       Author: Keyur Khambholja
*       Name: Stmsg
*       Description:
*       PreConditions:
*       PostCondions: 
*       Registers Modified: 
*       Calls:  None
********x*******x*******x*******x***********************
Stmsg   
	psha
	pshb
	
	ldaa    #%00011100
	cmpa    Flag+1
	bge     huh
	jsr     Inval
	bra     fini
huh     ldd     Recv
	anda    #%00001111
	std     Mes_num
	ldaa    #%10000000
	oraa    Flag+1
	staa    Flag+1
fini    pulb
	pula
	rts
;end Stmsg
********************************************************
*       Author: Keyur Khambholja
*       Name: Bdwdth
*       Description:
*       PreConditions:
*       PostCondions: 
*       Registers Modified: 
*       Calls:  None
********x*******x*******x*******x***********************
Bdwdth
	psha
	pshb
	ldd     Recv
	anda    #%00001111
	std     Bd
	bclr    PortD,x %011000         Set to move left
	bset    PortD,x %100000
	bclr    PortA,x %00100000

	ldd     Bd
	std     Dist
	jsr     Moveit

	ldd     #stck_sz        Transmit number of commands allowed
	anda    #%01111111
	oraa    #numall
	std     Trans
	jsr     Tx

	pulb
	pula
	rts
;end Bdwdth
********************************************************
*       Author: Keyur Khambholja
*       Name: Store
*       Description:  This routine will push a message onto the stack
*       PreConditions:  This routine acknowledges that we are ready 
*                       to receive messages
*       PostCondions: Counter is incremented 
*       Registers Modified: Stack2, Y, A
*       Calls:  None
********x*******x*******x*******x***********************
Store
	pshy
	psha
	sts     Stack1
	lds     Stack2
	ldy     Recv
	pshy
	sts     Stack2
	lds     Stack1
	ldy     Counter
	iny
	sty     Counter 
	pula    
	puly
	rts

;End Store
********************************************************
*       Author: Keyur Khambholja
*       Name: Invalid Receive
*       Description: A invalid message was received
*       PreConditions: An invalid message was received
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Inval
	psha
	pshb

	ldab    Flag+1          Clear out stack2 
	jsr     Cancel
	andb    #%00111111
	stab    Flag+1
		
	ldd     #invrcv         Inform PC of Invalid Status
	std     Trans
	jsr     Tx

	pulb
	pula

	rts
;end Inval
********************************************************
*       Author: Keyur Khambholja
*       Name: Process
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Process
	pshy
	psha
	pshb

	sts     Stack1
	lds     Stack2
  
pro     puly
	xgdy
	std     Message
	anda    #%10000000
	cmpa    #%10000000
	bne     notop
	jsr     Interp
	bra count

notop   ldaa    Message
	anda    #%00000111      
	cmpa    #$7
	bne     notprg
	ldd     #prgack
	std     Trans
	jsr     Tx
	bra     count

notprg  ldaa    Message
	anda    #%00001000
	cmpa    #%00001000
	bne     count
	jsr     Endop
       
count   ldy     Counter
	dey
	sty     Counter 
	bne     pro
	ldd     #stack2
	std     Stack2
	lds     Stack1
	clr     Counter
	clr     Counter+1

	pulb
	pula    
	puly
	
	rts
;end Process
********************************************************
*       Author: Keyur Khambholja
*       Name: Endop
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Endop
	ldd     #endopt
	std     Trans
	jsr     Tx
	
	rts
;end Endop
********************************************************
*       Author: Keyur Khambholja
*       Name: Interpret
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Interp
	pshx
	psha
	pshb
	
	ldx     #$1000

	bset    PortA,x %01100000       Turn Motors off
	ldd     Message
	anda    #%00000111
	std     Dist
	
	ldaa    Message                 Determine head Position
	anda    #head
	beq     up
	bset    PortD,x %010000
	bra     move
up      bclr    PortD,x %010000
move    ldaa    Message                 Determine movement direction
	anda    #movement
	cmpa    #right
	bne     notr                    Check for right movement
	bclr    PortD,x %100000
	bclr    PortA,x %00100000
	bra     moveme

notr    cmpa    #left
	bne     notl                    Check for left movement
	bclr    PortD,x %101000
	bset    PortD,x %100000
	bclr    PortA,x %00100000
	bra     moveme

notl    cmpa    #forward
	bne     notf                    Check for forward movement
	bset    PortD,x %001000
	bclr    PortA,x %01000000
	bra     moveme

notf    cmpa    #backward
	bne     nomove                  Check for backward movement
	bclr    PortD,x %001000         
	bclr    PortA,x %01000000

moveme  jsr     Moveit

nomove  pulb
	pula
	pulx
	rts
;end Interp
********************************************************
*       Author: Keyur Khambholja
*       Name: Move Itb
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Moveitb
	psha
	pshy
	pshx

	ldy     Dist

loopy   ldaa    PortE,x
	anda    #%00001111
	cmpa    #$F
	bne     lala
lala    ldx     #milval
	jsr     Twomil
	dey
	bne     loopy
	jsr     Motoroff
	bra goo
goo     pulx
	puly
	pula
	rts
;end Moveitb

********************************************************
*       Author: Keyur Khambholja
*       Name: Move It
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Moveit
	psha
	pshy
	pshx

	ldy     Dist

loopx   ldaa    PortE,x
	anda    #%00001111
	cmpa    #$F
	bne     limiter
	ldx     #milval
	jsr     Twomil
	dey
	bne     loopx
	jsr     Motoroff
	bra good
limiter jsr     Hit
good    pulx
	puly
	pula
	rts
;end Moveit
********************************************************
*       Author: Keyur Khambholja
*       Name: Motor Off
*       Description: Turn Motors Off
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Motoroff
	pshx
	ldx     #$1000
	bset    PortA,x %01100000        
	pulx
	rts
;end Motoroff
********************************************************
*       Author: Keyur Khambholja
*       Name: Two Mill
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Twomil        
milloop dex
	bne     milloop
	rts
;end Moveit
********************************************************
*       Author: Keyur Khambholja
*       Name: Hit
*       Description: Limiter hit
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Hit
	pshy
	pshb
	pshx
	ldx     #$1000

	jsr     Motoroff
	ldab    PortE,x
	andb    #%00001111
	ldy     #lim_tb
	aby
	aby
	ldy     0,y
	jsr     0,y

	pulb
	puly
	rts
;end Hit
********************************************************
*       Author: Keyur Khambholja
*       Name: Nouse
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Nouse
	rts
;end Nouse
********************************************************
*       Author: Keyur Khambholja
*       Name: Xmin
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Xmin    
	ldd     #xminlim
	std     Trans
	jsr     Tx
	bclr    PortD,x %100000
	bclr    PortA,x %00100000
	ldy     #$5
	sty     Dist
	jsr     Moveitb
	rts
;end Xmin
********************************************************
*       Author: Keyur Khambholja
*       Name: Xmax
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Xmax
	ldd     #ymaxlim
	std     Trans
	jsr     Tx
	bclr    PortD,x %101000
	bset    PortD,x %100000
	bclr    PortA,x %00100000
	ldy     #$5
	sty     Dist
	jsr     Moveitb
	rts
;end Xmax
********************************************************
*       Author: Keyur Khambholja
*       Name: Ymin
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Ymin
	ldd     #yminlim
	std     Trans
	jsr     Tx
	bset    PortD,x %001000
	bclr    PortA,x %01000000
	ldy     #$5
	sty     Dist
	jsr     Moveitb
	rts
;end Ymin
********************************************************
*       Author: Keyur Khambholja
*       Name: Ymax
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Ymax
	ldd     #ymaxlim
	std     Trans
	bclr    PortD,x %001000         
	bclr    PortA,x %01000000
	ldy     #$5
	sty     Dist
	jsr     Moveitb
	jsr     Tx
	rts
;end Ymax
********************************************************
*       Author: Keyur Khambholja
*       Name: Transmit
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
Tx      
	pshx
	psha
	pshb
	
	ldx     #$1000 
	ldab    #sccr2b
	stab    SCCR2,x
	brclr   SCSR,x %10000000  *
	ldaa    Trans
	ldab    SCSR,x
	staa    SCDR,x
	brclr   SCSR,x %10000000  *
	ldaa    Trans+1
	ldab    SCSR,x
	staa    SCDR,x
	ldab    #sccr2
	stab    SCCR2,x

	pulb
	pula
	pulx
	rts
;end Tx
********************************************************
*       Author: Keyur Khambholja
*       Name: Recieve Interrupt
*       Description:
*       PreConditions:
*       PostCondions:
*       Registers Modified:
*       Calls:
********x*******x*******x*******x***********************
rcv_int 
	sei                     Inhibit all interrupts

	ldx     #$1000
		
	ldd     Flag            Test to see if first half is received
	andb    #rcv_buf1
	bne     sechalf
*
	ldd     Flag            Set bit 0 high to indicate first half received
	orab    #rcv_buf1
	stab    Flag+1
*
	clr     Recv
	clr     Recv+1
*
	ldaa    SCSR,x
	ldaa    SCDR,x          Read RDR and store in low
	staa    Recv                 portion of Receive byte
*
	bra     end_buf         Reenable interrupts and exit interrupt

sechalf 
	ldd     Flag            Set bit 1 high to indicate first half received
	orab    #rcv_buf2
	stab    Flag+1
*
	ldaa    SCSR,x
	ldaa    SCDR,x          Read RDR and store in high
	staa    Recv+1                 portion of Receive byte

end_buf cli
	rti
;end Rcv_int

********x*******x*******x*******x***********************
*       Reserved memory 
********x*******x*******x*******x***********************
Counter rmb     2
Flag    rmb     2
Recv    rmb     2
Stack1  rmb     2
Stack2  rmb     2
Trans   rmb     2
Bd      rmb     2
Mes_num rmb     2
Dist    rmb     2
Message rmb     2

recv_tb fdb     Error
	fdb     Cancel
	fdb     Manual
	fdb     Auto
	fdb     Sttrm
	fdb     Aytmsg
	fdb     Endmsg
	fdb     Progrcv
	fdb     Endopr
	fdb     Calib
	fdb     Eject

lim_tb  fdb     Nouse
	fdb     Nouse
	fdb     Nouse
	fdb     Nouse
	fdb     Nouse
	fdb     Nouse
	fdb     Nouse
	fdb     Xmax
	fdb     Nouse
	fdb     Nouse
	fdb     Nouse
	fdb     Xmin
	fdb     Nouse
	fdb     Ymax
	fdb     Ymin
