///////////////////////////////////////////////////////////////
// File Name: basetypes.h
// Purpose: define some basic data types used by the simulator
///////////////////////////////////////////////////////////////

#ifndef BASETYPE_H 
#define BASETYPE_H

// boolean
// typedef unsigned char bool;
// 1 bit (bool is inefficient)
typedef unsigned char bit_t;
// 8 bits
typedef unsigned char byte_t;
// 16 bits
typedef unsigned short word_t;
// 32 bits
typedef unsigned long dword_t;

//////////////////////////////////////////////////////////////////////////////
//
// Class Name: Byte
//
// Author: Kalle Anderson
//
// Purpose: Allows access to the bits of a byte(unsinged char).Bit-fields
//          were not used for this, because the C/C++ standard does not
//          gaurantee the ordering of bits in a bit-field.
//
//          The bit_t structure could have been a bool, but the compiler
//          would have generated extra code in order to gaurantee that the
//          value was always 0 or 1.
//
/////////////////////////////////////////////////////////////////////////////
struct Byte
{
    byte_t byte;    

    bit_t bit0(void){return (byte&0x1);};
    bit_t bit1(void){return (byte&0x2)>>1;};
    bit_t bit2(void){return (byte&0x4)>>2;};
    bit_t bit3(void){return (byte&0x8)>>3;};
    bit_t bit4(void){return (byte&0x10)>>4;};
    bit_t bit5(void){return (byte&0x20)>>5;};
    bit_t bit6(void){return (byte&0x40)>>6;};
    bit_t bit7(void){return (byte&0x80)>>7;};

    void bit0(bit_t value){byte = (byte & 0xfe) | value;};
    void bit1(bit_t value){byte = (byte & 0xfd) | value<<1;};
    void bit2(bit_t value){byte = (byte & 0xfb) | value<<2;};
    void bit3(bit_t value){byte = (byte & 0xf7) | value<<3;};
    void bit4(bit_t value){byte = (byte & 0xef) | value<<4;};
    void bit5(bit_t value){byte = (byte & 0xdf) | value<<5;};
    void bit6(bit_t value){byte = (byte & 0xbf) | value<<6;};
    void bit7(bit_t value){byte = (byte & 0x7f) | value<<7;};

    Byte(void){};
    Byte(byte_t data){byte = data;};
	operator byte_t(){return byte;}; 
	Byte &operator=(byte_t data){byte = data;return *this;};
};


//////////////////////////////////////////////////////////////////////////////
//
// Class Name: Byte
//
// Author: Kalle Anderson
//
// Purpose: Allows access to the bits and bytes of a word. (see Byte class)
//
/////////////////////////////////////////////////////////////////////////////


struct Word
{
    word_t word;

    byte_t low(void){return word&0x00FF;};
    byte_t high(void){return (word&0xFF00)>>8;};
    void low(byte_t value){word = (word&0xFF00)|value;};
    void high(byte_t value){word = (word&0x00FF)|(value<<8);};

    bit_t bit0(void){return (word&0x1);};
    bit_t bit1(void){return (word&0x2)>>1;};
    bit_t bit2(void){return (word&0x4)>>2;};
    bit_t bit3(void){return (word&0x8)>>3;};
    bit_t bit4(void){return (word&0x10)>>4;};
    bit_t bit5(void){return (word&0x20)>>5;};
    bit_t bit6(void){return (word&0x40)>>6;};
    bit_t bit7(void){return (word&0x80)>>7;};
    bit_t bit8(void){return (word&0x100)>>8;};
    bit_t bit9(void){return (word&0x200)>>9;};
    bit_t bit10(void){return (word&0x400)>>10;};
    bit_t bit11(void){return (word&0x800)>>11;};
    bit_t bit12(void){return (word&0x1000)>>12;};
    bit_t bit13(void){return (word&0x2000)>>13;};
    bit_t bit14(void){return (word&0x4000)>>14;};
    bit_t bit15(void){return (word&0x8000)>>15;};

    void bit0(bit_t value){word = (word & 0xfffe) | (word_t)value;};
    void bit1(bit_t value){word = (word & 0xfffd) | (word_t)value<<1;};
    void bit2(bit_t value){word = (word & 0xfffb) | (word_t)value<<2;};
    void bit3(bit_t value){word = (word & 0xfff7) | (word_t)value<<3;};
    void bit4(bit_t value){word = (word & 0xffef) | (word_t)value<<4;};
    void bit5(bit_t value){word = (word & 0xffdf) | (word_t)value<<5;};
    void bit6(bit_t value){word = (word & 0xffbf) | (word_t)value<<6;};
    void bit7(bit_t value){word = (word & 0xff7f) | (word_t)value<<7;};
    void bit8(bit_t value){word = (word & 0xfeff) | (word_t)value<<8;};
    void bit9(bit_t value){word = (word & 0xfdff) | (word_t)value<<9;};
    void bit10(bit_t value){word = (word & 0xfbff) | (word_t)value<<10;};
    void bit11(bit_t value){word = (word & 0xf7ff) | (word_t)value<<11;};
    void bit12(bit_t value){word = (word & 0xefff) | (word_t)value<<12;};
    void bit13(bit_t value){word = (word & 0xdfff) | (word_t)value<<13;};
    void bit14(bit_t value){word = (word & 0xbfff) | (word_t)value<<14;};
    void bit15(bit_t value){word = (word & 0x7fff) | (word_t)value<<15;};


    operator word_t(){return word;}; 
    Word operator=(word_t data){word = data;return *this;};    

    Word(void){};
    Word(word_t data){word = data;};
    Word &operator++(int){word++;return *this;};
    Word &operator--(int){word--;return *this;};
};


#endif
